/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.log.ui.logdialog;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class LogPreviewUI extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	protected XLog log;

	protected JList instancesList;
	protected JLabel instanceNameLabel;
	protected JLabel instanceSizeLabel;
	protected JList eventsList;
	protected JLabel eventLabel;
	protected JList attributesList;

	public LogPreviewUI(XLog log, XLogInfo summary) {
		this.log = log;
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		// create instances list
		instancesList = new JList();
		instancesList.setBackground(COLOR_LIST_BG);
		instancesList.setForeground(COLOR_LIST_FG);
		instancesList.setSelectionBackground(COLOR_LIST_SELECTION_BG);
		instancesList.setSelectionForeground(COLOR_LIST_SELECTION_FG);
		instancesList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		instancesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				instancesSelectionChanged();
			}
		});

		String[] instanceNames = new String[log.size()];
		for (int i = 0; i < instanceNames.length; i++) {
			instanceNames[i] = XConceptExtension.instance().extractName(log.get(i));
		}
		instancesList.setListData(instanceNames);
		// reset events list
		instancesList.clearSelection();

		JScrollPane instancesScrollPane = new JScrollPane(instancesList);
		instancesScrollPane.setOpaque(false);
		//instancesScrollPane.setViewportBorder(BorderFactory.createLineBorder(colorListBorder));
		instancesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		instancesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		instancesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JScrollBar vBar = instancesScrollPane.getVerticalScrollBar();
		vBar
				.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS,
						4, 12));
		vBar.setOpaque(false);
		// assemble instances list
		JLabel instancesListLabel = new JLabel("Instances");
		instancesListLabel.setOpaque(false);
		instancesListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		instancesListLabel.setFont(instancesListLabel.getFont().deriveFont(13f));
		instancesListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instancesListLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instancesListLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		RoundedPanel instancesPanel = new RoundedPanel(10, 5, 0);
		instancesPanel.setBackground(COLOR_ENCLOSURE_BG);
		instancesPanel.setLayout(new BoxLayout(instancesPanel, BoxLayout.Y_AXIS));
		instancesPanel.setMaximumSize(new Dimension(180, 1000));
		instancesPanel.setPreferredSize(new Dimension(180, 1000));
		instancesPanel.add(instancesListLabel);
		instancesPanel.add(Box.createVerticalStrut(8));
		instancesPanel.add(instancesScrollPane);
		// create event list header
		instanceNameLabel = new JLabel("(no instance selected)");
		instanceNameLabel.setOpaque(false);
		instanceNameLabel.setForeground(COLOR_NON_FOCUS);
		instanceNameLabel.setFont(instanceNameLabel.getFont().deriveFont(13f));
		instanceNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instanceNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instanceNameLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		instanceSizeLabel = new JLabel("select single instance to browse");
		instanceSizeLabel.setOpaque(false);
		instanceSizeLabel.setForeground(COLOR_NON_FOCUS);
		instanceSizeLabel.setFont(instanceSizeLabel.getFont().deriveFont(11f));
		instanceSizeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instanceSizeLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instanceSizeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		// create events list
		eventsList = new JList();
		eventsList.setBackground(COLOR_LIST_BG);
		eventsList.setCellRenderer(new EventCellRenderer());
		eventsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		eventsList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				eventsSelectionChanged();
			}
		});
		JScrollPane eventsScrollPane = new JScrollPane(eventsList);
		eventsScrollPane.setOpaque(false);
		eventsScrollPane.setBorder(BorderFactory.createEmptyBorder());
		//eventsScrollPane.setViewportBorder(BorderFactory.createLineBorder(colorListBorder));
		eventsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		eventsScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		vBar = eventsScrollPane.getVerticalScrollBar();
		vBar
				.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160), COLOR_NON_FOCUS,
						4, 12));
		vBar.setOpaque(false);
		// assemble events list
		RoundedPanel eventsPanel = new RoundedPanel(10, 5, 0);
		eventsPanel.setBackground(COLOR_ENCLOSURE_BG);
		eventsPanel.setLayout(new BoxLayout(eventsPanel, BoxLayout.Y_AXIS));
		eventsPanel.add(instanceNameLabel);
		eventsPanel.add(instanceSizeLabel);
		eventsPanel.add(Box.createVerticalStrut(8));
		eventsPanel.add(eventsScrollPane);
		// create attributes list
		attributesList = new JList();
		attributesList.setBackground(COLOR_LIST_BG);
		attributesList.setForeground(COLOR_LIST_FG);
		attributesList.setSelectionBackground(COLOR_LIST_BG);
		attributesList.setSelectionForeground(COLOR_LIST_FG);
		attributesList.setFocusable(false);
		JScrollPane attributesScrollPane = new JScrollPane(attributesList);
		attributesScrollPane.setOpaque(false);
		//attributesScrollPane.setBackground(colorListBg);
		attributesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		//attributesScrollPane.setViewportBorder(BorderFactory.createLineBorder(colorListBorder));
		attributesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		attributesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		vBar = attributesScrollPane.getVerticalScrollBar();
		vBar
				.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS,
						4, 12));
		vBar.setOpaque(false);
		vBar = attributesScrollPane.getHorizontalScrollBar();
		vBar
				.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(200, 200, 200), COLOR_NON_FOCUS,
						4, 12));
		vBar.setOpaque(false);
		// assemble attributes panel
		eventLabel = new JLabel("(no event selected)");
		eventLabel.setOpaque(false);
		eventLabel.setForeground(COLOR_NON_FOCUS);
		eventLabel.setFont(eventLabel.getFont().deriveFont(13f));
		eventLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		eventLabel.setHorizontalAlignment(SwingConstants.CENTER);
		eventLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		RoundedPanel attributesPanel = new RoundedPanel(10, 5, 0);
		attributesPanel.setBackground(COLOR_ENCLOSURE_BG);
		attributesPanel.setLayout(new BoxLayout(attributesPanel, BoxLayout.Y_AXIS));
		attributesPanel.setMaximumSize(new Dimension(230, 1000));
		attributesPanel.setPreferredSize(new Dimension(230, 1000));
		attributesPanel.add(eventLabel);
		attributesPanel.add(Box.createVerticalStrut(8));
		attributesPanel.add(attributesScrollPane);
		// assemble GUI
		this.add(instancesPanel);
		this.add(eventsPanel);
		this.add(attributesPanel);
	}

	protected void instancesSelectionChanged() {
		int[] selectedIndices = instancesList.getSelectedIndices();
		if ((selectedIndices.length == 0) || (selectedIndices.length > 1)) {
			eventsList.setListData(new Object[] {});
			eventsList.clearSelection();
			instanceNameLabel.setForeground(COLOR_NON_FOCUS);
			instanceNameLabel.setText("(no instance selected)");
			instanceSizeLabel.setForeground(COLOR_NON_FOCUS);
			instanceSizeLabel.setText("select single instance to browse");
		} else {
			XTrace instance = log.get(selectedIndices[0]);
			instanceNameLabel.setForeground(COLOR_LIST_SELECTION_FG);
			instanceNameLabel.setText(XConceptExtension.instance().extractName(instance));
			instanceSizeLabel.setForeground(COLOR_LIST_SELECTION_FG);
			instanceSizeLabel.setText(instance.size() + " events");
			eventsList.setModel(new XTraceListModel(instance));
			eventsList.ensureIndexIsVisible(0);
		}
		showSelectedInstanceData();
	}

	protected void eventsSelectionChanged() {
		XEvent ate = (XEvent) eventsList.getSelectedValue();

		if (ate != null) {
			eventLabel.setForeground(COLOR_LIST_SELECTION_FG);
			eventLabel.setText("Attributes for event " + (eventsList.getSelectedIndex() + 1));
			attributesList.setModel(new AttributesListModel(ate));
		} else {
			showSelectedInstanceData();
		}
	}

	protected void showSelectedInstanceData() {
		int[] selectedIndices = instancesList.getSelectedIndices();

		if (selectedIndices.length == 1) {
			XTrace instance = log.get(selectedIndices[0]);

			eventLabel.setForeground(COLOR_LIST_SELECTION_FG);
			eventLabel.setText("Attributes for case " + XConceptExtension.instance().extractName(instance));
			attributesList.setModel(new AttributesListModel(instance));
		} else {
			eventLabel.setForeground(COLOR_NON_FOCUS);
			eventLabel.setText("(no case or event selected)");
			attributesList.setModel(new AttributesListModel((XTrace) null));
		}
	}

}
