/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.log.ui.logdialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.deckfour.uitopia.ui.util.ImageLoader;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.log.logfilters.LogFilter;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class SlickerOpenLogSettings extends JPanel {

	private static final long serialVersionUID = 4258730373866637365L;

	protected InspectorUI inspectorUI;

	// icons
	protected Image dashboardIcon = (new ImageIcon("images/slicker/dashboard48.png")).getImage();
	protected Image inspectorIcon = (new ImageIcon("images/slicker/inspector48.png")).getImage();
	protected Image summaryIcon = (new ImageIcon("images/slicker/summary48-2.png")).getImage();

	// data attributes
	protected XLog log;
	protected LogFilter currentFilter;
	protected XLogInfo summary;

	protected SlickerTabbedPane tabbedPane;

	@Plugin(name = "Log Summary", parameterLabels = "Log", returnTypes = JComponent.class, returnLabels = "Log Visualization", userAccessible = false, mostSignificantResult = 1, help = "Provides an overview of the log")
	@Visualizer(name = "Log Visualizer")
	public JComponent showLogVis(UIPluginContext context, XLog log) {
		this.log = log;
		summary = XLogInfoFactory.createLogInfo(log);
		initialize();
		completeGui(context);
		return this;
	}

	/**
	 * loads the log and initializes this component
	 */
	protected void initialize() {
		String promDir = System.getProperty("user.dir");
		if (promDir.endsWith(".")) {
			promDir = promDir.substring(0, promDir.length() - 1);
		}
		if (promDir.endsWith(System.getProperty("file.separator")) == false) {
			promDir += System.getProperty("file.separator");
		}
		dashboardIcon = ImageLoader.load("dashboard48.png");
		inspectorIcon = ImageLoader.load("inspector48.png");
		summaryIcon = ImageLoader.load("summary48-2.png");

		setBackground(new Color(40, 40, 40));

		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());

	}

	protected void completeGui(UIPluginContext context) {
		// compile inspector panel
		inspectorUI = new InspectorUI(log, summary);
		// compile root panel
		IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,
				20, 160));
		iconTabs.addTab("Dashboard", dashboardIcon, new LogDashboardUI(log, summary));
		iconTabs.addTab("Inspector", inspectorIcon, inspectorUI);
		iconTabs.addTab("Summary", summaryIcon, new LogInfoUI(this, context));
		add(iconTabs, BorderLayout.CENTER);
		revalidate();
		repaint();
	}

	public XLog getLog() {
		return log;
	}

	public XLogInfo getLogInfo() {
		return summary;
	}

}
